import os
from sys import stdout
from time import sleep
from utils import thread_helpers
from termcolor import colored



class animate_iteration:
    def __init__(self, animationList: list, static: str, prefix=None, completion=None, delay=0.2, textColour=None):
        """
        :param animationList:
        Any list of string will be iterated through
        if the animation list is set to ['default'] automatically use ['/', '-', '\\', '|']
        :param static:
        prints right before animation list
        :param prefix:
        prints before static
        :param completion:
        prints after .stop_animation
        :param delay:
        delay between print
        :param textColour:
        color of the animation
        available colors ['red', 'green', 'yellow', 'blue', 'magenta', 'cyan', 'white']
        """
        if animationList[0] == "default":
            animationList = ['/', '-', '\\', '|']

        self.animation = animationList
        self.delay = delay
        self.static = static
        self.prefix = prefix
        self.stop = False
        self.completion = completion
        self.colors = textColour
        self._mutate_animation()

    def _mutate_animation(self, completion=False):
        if self.colors is not None:
            self.static = colored(text=self.static, color=self.colors)

            if self.prefix is not None:
                self.prefix = colored(text=self.prefix, color=self.colors)

            if self.completion is not None and completion:
                self.completion = colored(text=self.completion, color=self.colors)

    def start_animation(self):
        def animate(selfClass):
            while True:
                self._mutate_animation()
                if self.stop:
                    if self.completion is not None:
                        self._clear_screen()
                        self._mutate_animation(completion=True)

                        try:
                            if self.prefix is None:
                                self.prefix = ''
                        except TypeError:
                            self.prefix = ''

                        print('\r' + self.prefix, self.completion)
                    break

                for animation in selfClass.animation:
                    if self.prefix is not None:
                        stdout.write('\r' + self.prefix + ' ' + selfClass.static + ' ' + animation)
                    else:
                        stdout.write('\r' + selfClass.static + ' ' + animation)

                    stdout.flush()
                    sleep(selfClass.delay)


                    try:
                        stdout.write('\r' + str(' ' * os.get_terminal_size()[0]))
                    except OSError:
                        if self.prefix is not None:
                            stdout.write('\r' + ' ' * len(self.prefix + ' ' + selfClass.static + ' ' + animation))
                        else:
                            stdout.write('\r' + ' ' * len(selfClass.static + ' ' + animation))


        thread_helpers.thread(func=animate, args=[self])

    def stop_animation(self):
        self.stop = True
        sleep(1)

    def change_static(self, static: str):
        self._clear_screen()
        self.static = static
        self._mutate_animation()

    def _clear_screen(self):
        try:
            stdout.write('\r' + str(' ' * os.get_terminal_size()[0]))
        except:
            stdout.write('\r' + ' ' * len(self.static))

    def change_animation_list(self, animationList: list):
        self.animation = animationList

    def change_prefix(self, prefix: str):
        self.prefix = prefix
        self._mutate_animation()

    def change_color(self, color):
        self.colors = color


if __name__ == '__main__':
    # for debugging
    # animations = ['/', '-', '\\', '|']
    # a = animate_iteration(animationList=animations, static='Animatingggggg',
    #                       completion='Hello Bye', prefix='[Static]',
    #                       textColour='red')
    # a.start_animation()
    # sleep(1)
    #
    # a.change_color(color='green')
    # a.change_prefix(prefix='[SOOO]')
    # a.change_static('FOoo')
    # a.change_animation_list(animationList=['a', 'b', 'c'])
    #
    # sleep(3)
    #
    # a.stop_animation()
    #
    # sleep(1)
    #
    # print('end')
    pass



import requests


class requests_session:
    def __init__(self, optional_func=None, header=None):
        self.ses = requests.Session()
        self.function = optional_func
        self.const_head = header

    def get(self, url, headers=None):
        if headers is None:
            headers = self.const_head

        response = self.ses.get(url, headers=headers)
        if self.function is None:
            pass
        else:
            self.function(response)

        return response

    def post(self, url, data=None, json=None, headers=None):
        if headers is None:
            headers = self.const_head

        request = self.ses.post(url=url, data=data, json=json, headers=headers)
        if self.function is not None:
            self.function(request)

        return request

    def session(self):
        return self.ses

    def request(self, method, url, params=None, data=None, headers=None,
                cookies=None, files=None, auth=None, timeout=None,
                allow_redirects=True, proxies=None,
                hooks=None, verify=None,
                json=None):

        if headers is None:
            headers = self.const_head

        request = self.ses.request(url=url, method=method, proxies=proxies,
                                   params=params, data=data, headers=headers,
                                   cookies=cookies, files=files, auth=auth, timeout=timeout,
                                   allow_redirects=allow_redirects, hooks=hooks, verify=verify, json=json)

        if self.function is not None:
            self.function(request)

        return request

    def change_optional_func(self, function):
        self.function = function

    def change_constant_header(self, new_header):
        self.const_head = new_header


def waste():
    pass



import threading
from utils.thread_helpers.animations import animate_iteration

def thread(func, args=None, daemon=True):
    '''
    Must be a function in without ()
    example if the function is def hello():
    thread would be called like so thread(func=hello, args=['argument1', 'argument2']
    '''
    if args is None:
        args = []

    Obj = threading.Thread(target=func, args=args, daemon=daemon)
    Obj.start()
    return Obj

def multiple_threads(function_data, daemon=True):
    """
    Functions must be in a list like so [function1, function2, function3]
    :param daemon:
    true by default
    :param function_data:

    List of Dictionary like so
    function_data = [
        {
        'function' : function
        'args' : [arg1, arg2]
        }
    ]

    :return:
    A list of all the started threads
    """
    x = 0
    Objs = []
    for f in function_data:
        Args = f['args']
        if Args == None:
            Args = []

        Function = f['function']
        Objs.append(threading.Thread(target=Function, args=Args, daemon=daemon))


    for o in Objs:
        o.start()


    return Objs




import signal
import subprocess
import os
from . import paths

def HomeDirectory():
    return os.path.expanduser('~')

def command(args: list, quite=False, read=False):
    if quite:
        sub = subprocess.Popen(args, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)

    elif read:
        sub = subprocess.Popen(args, stdout=subprocess.PIPE, stdin=subprocess.PIPE,
                               stderr=subprocess.STDOUT)

        response = sub.communicate()[0].decode('utf8')
        sub.wait()
        sub.poll()
        returnCode = int(sub.returncode)

        return response, returnCode, sub
    else:
        sub = subprocess.Popen(args)

    sub.wait()
    sub.kill()
    sub.terminate()

def ip_address(interface='en0'):
    ip = command(['ipconfig', 'getifaddr', interface], read=True)
    try:
        ip = ip[0].removesuffix('\n')
    except AttributeError or Exception:
        if ip[0].endswith('\n'):
            ip = ip[0][:-1]

    return ip

def hostname():
    import socket
    return socket.gethostname()

def remove(file):
    command(['rm', '-rf', file])

def kill_process(pid):
    if not isinstance(pid, int):
        pid = int(pid)
    os.kill(pid, signal.SIGTERM)



if __name__ == '__main__':
    pass



import json
def write_new_js(data, file_name):
    with open('{}.json'.format(file_name), 'w+') as file:
        file.seek(0)
        json.dump(data, file, indent=4)


def open_load_file(file_name):
    file = open('{}.json'.format(file_name), 'r+')
    return json.load(file), file


def update_dict(data, file_name):
    with open('{}.json'.format(file_name), 'r+') as file:
        JsF = json.load(file)
        JsF.update(data)

        file.seek(0)
        json.dump(JsF, file, indent=4)
        file.close()

def dump_js_to_file(file, Js):
    file.seek(0)
    json.dump(Js, file, indent=4)

def append_to_list(list_key, file_name, list_data_append):
    s = open_load_file(file_name)

    Js = s[0]
    file = s[1]

    Js[list_key].append(list_data_append)
    dump_js_to_file(file=file, Js=Js)


def get_values_list(file_name, key=None):
    s = open_load_file(file_name)
    Js = s[0]

    if key is not None:
        return Js[key]
    else:
        return Js

def waste():
    pass


import setuptools

Req = ['requests',
       'termcolor']

version = '3.0'

setuptools.setup(
    name='utils-s',
    version=version,
    author="Sal Faris",
    description="Utility functions",
    packages=setuptools.find_packages(),
    url='https://github.com/The-Sal/utils/',
    install_requires=Req
)



import json
from os import path
from utils.system.paths import Path
from utils.exceptions.Js_exception import badJSON

class JsHandler:
    def __init__(self, file_name):
        if isinstance(file_name, Path):
            self.file = file_name.path
        else:
            self.file = file_name

        if path.isfile(self.file):
            self.__loadFile()
        else:
            with open(self.file, 'w+') as f:
                json.dump({}, f, indent=4)

    def __loadFile(self) -> dict:
        fl = open(self.file, 'r+')
        return json.load(fl)

    def __dump(self, newDict):
        try:
            json.dumps(newDict)
        except Exception:
            raise badJSON('Unable to convert dictionary into JSON data')

        with open(self.file, 'w+') as file:
            json.dump(newDict, file, indent=4)

    def __getitem__(self, item):
        internal_json_dict = self.__loadFile()
        requestItem = internal_json_dict[item]
        return requestItem

    def __enter__(self):
        return self.__loadFile()

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    def update(self, updatedDict):
        Js = self.__loadFile()
        Js.update(updatedDict)
        self.__dump(Js)

    def getDict(self) -> dict:
        return self.__loadFile()

    def keys(self):
        return self.__loadFile().keys()

    def pop(self, key):
        Js = self.__loadFile()
        Js.pop(key)
        self.__dump(newDict=Js)



import os
import random
import shutil
import subprocess
from utils import exceptions


def __command(args: list):
    sub = subprocess.Popen(args, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
    sub.wait()
    sub.kill()
    sub.terminate()


def _decoder(string_path: str) -> str:
    if string_path.startswith('~'):
        string_path = join(os.path.expanduser('~'), string_path.split('~', 1)[1])

    if not string_path.__contains__('\\'):
        return string_path

    string_path = string_path.replace('\\', '')

    return string_path


def last_component(string: str):
    Broken = string.split('/')
    length = len(Broken)

    return Broken[length - 1]


def basePath(URL):
    if URL.endswith('/'):
        URL = URL[:-1]

    Broken = URL.split('/')
    length = len(Broken)

    LastItem = Broken[length - 1]
    NewPath = ''

    for path in Broken:
        Bad = [LastItem, '']

        continuee = False

        for b in Bad:
            if path == b:
                continuee = True

        if continuee:
            continue

        NewPath = NewPath + '/' + path

    return NewPath


def file_exists(path):
    return os.path.isfile(path)


def directory_exists(path):
    return os.path.isdir(path)


def os_walk(file_name, path, custom_function=None):
    """
    custom_function is a function which should return True
    if the file passed is the wanted file

    i.e.
    def function(file):
        if file.endswith('.wav'):
        return True

    in this example the first file that ends with a .wav extension will be returned
    """
    if custom_function is None:
        for r, d, f in os.walk(path):
            for file in f:
                if file == file_name:
                    return os.path.abspath(os.path.join(r, file))

    else:
        for r, d, f in os.walk(path):
            for file in f:
                if custom_function(file):
                    return os.path.abspath(os.path.join(r, file))

        return None


def copy(src, dst):
    if os.path.isdir(src):
        if dst.endswith('/'):
            dst = dst + last_component(src)
        elif not dst.endswith('/'):
            dst = dst + '/' + last_component(src)
        return shutil.copytree(src=src, dst=dst)
    else:
        return shutil.copy(src=src, dst=dst)


def join(path1: str, path2: str):
    if path1.endswith('/'):
        path1 = path1[:-1]

    if path2.endswith('/'):
        path2 = path2[:-1]

    if not path1.startswith('/'):
        path1 = '/' + path1

    if not path2.startswith('/'):
        path2 = '/' + path2

    path = path1 + path2
    return path


def remove(file):
    # pure python way to remove files
    if os.path.isdir(file):
        shutil.rmtree(file)
    else:
        os.remove(file)


class Path:
    def __init__(self, path):
        path = _decoder(path)

        if not path.startswith('/'):
            path = '/' + path

        if path.endswith('/'):
            try:
                path = path.removesuffix('/')
            except AttributeError or Exception:
                path = path[:-1]

        if path.endswith(' '):
            try:
                path = path.removesuffix(' ')
            except AttributeError or Exception:
                path = path[:-1]

        self.path = path

    def append(self, path2):
        return Path(join(self.path, path2))

    def last_component(self):
        return last_component(self.path)

    def rename(self, name):
        src = self.path
        dst = join(basePath(self.path), '/' + name)
        os.rename(src=src, dst=dst)
        self.path = dst

        return Path(self.path)

    def write_to(self, WritingData, OpenMode):
        with open(self.path, OpenMode) as file:
            file.write(WritingData)

    def read_file(self, OpenMode):
        with open(self.path, OpenMode) as file:
            file_read = file.read()

        return file_read

    def isPath(self):
        if file_exists(self.path) or directory_exists(self.path):
            return True
        else:
            return False

    def basePath(self):
        return basePath(self.path)

    def __abs__(self):
        return self.path

    def __str__(self):
        return abs(self)

    def __len__(self):
        if directory_exists(self.path):
            return len(os.listdir(self.path))
        elif file_exists(self.path):
            return 1
        else:
            raise FileNotFoundError(self.path)

    def __add__(self, other):
        return abs(self) + other

    def __bool__(self):
        return self.isPath()

    def __iter__(self):
        if self.isPath():
            if directory_exists(abs(self)):
                return iter(os.listdir(self.path))
            else:
                return iter([])
        else:
            raise FileNotFoundError(self.path)


class container:
    def __init__(self):
        self.old_cwd = os.getcwd()
        self.file = Path(join(self.old_cwd, random.random().__str__()))

    def __enter__(self):
        os.mkdir(abs(self.file))
        return self.file

    def __exit__(self, exc_type, exc_val, exc_tb):
        remove(self.file)
        os.chdir(self.old_cwd)


# Depreciated Functions
def remove_file(file, blank):
    # _command(['rm', '-rf', file])
    raise exceptions.DepreciatedFunction(__file__, 'remove_file')


if __name__ == '__main__':
    pass



# global unspecific exceptions
class utilityErrors(Exception):
    pass

class pathError(utilityErrors):
    pass

class Depreciated(utilityErrors):
    pass

class DepreciatedFunction(Exception):
    def __init__(self, source, function_name):
        errmsg = """The function .{}() from {} is deprecated""".format(function_name, source)
        raise Depreciated(errmsg)
